@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -27,16 +27,20 @@
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <errno.h>
 
 #include <glib.h>
 
 #include <ofono/log.h>
 #include <ofono/modem.h>
 #include <ofono/voicecall.h>
+#include "vendor.h"
 
 #include "gatchat.h"
 #include "gatresult.h"
 
+#include  "common.h"
+
 #include "atmodem.h"
 
 /* Amount of ms we wait between CLCC calls */
@@ -45,17 +49,28 @@
  /* Amount of time we give for CLIP to arrive before we commence CLCC poll */
 #define CLIP_INTERVAL 200
 
+ /* When +VTD returns 0, an unspecified manufacturer-specific delay is used */
+#define TONE_DURATION 1000
+
 static const char *clcc_prefix[] = { "+CLCC:", NULL };
 static const char *none_prefix[] = { NULL };
 
 /* According to 27.007 COLP is an intermediate status for ATD */
 static const char *atd_prefix[] = { "+COLP:", NULL };
 
+#define FLAG_NEED_CLIP 1
+#define FLAG_NEED_CNAP 2
+
 struct voicecall_data {
 	GSList *calls;
 	unsigned int local_release;
 	unsigned int clcc_source;
 	GAtChat *chat;
+	unsigned int vendor;
+	unsigned int tone_duration;
+	guint vts_source;
+	unsigned int vts_delay;
+	unsigned char flags;
 };
 
 struct release_id_req {
@@ -97,8 +112,7 @@ static struct ofono_call *create_call(struct ofono_voicecall *vc, int type,
 
 	/* Generate a call structure for the waiting call */
 	call = g_try_new0(struct ofono_call, 1);
-
-	if (!call)
+	if (call == NULL)
 		return NULL;
 
 	call->id = ofono_voicecall_get_next_callid(vc);
@@ -113,6 +127,7 @@ static struct ofono_call *create_call(struct ofono_voicecall *vc, int type,
 	}
 
 	call->clip_validity = clip;
+	call->cnap_validity = CNAP_VALIDITY_NOT_AVAILABLE;
 
 	d->calls = g_slist_insert_sorted(d->calls, call, at_util_call_compare);
 
@@ -128,8 +143,6 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_call *nc, *oc;
 	gboolean poll_again = FALSE;
 
-	dump_response("clcc_poll_cb", ok, result);
-
 	if (!ok) {
 		ofono_error("We are polling CLCC and received an error");
 		ofono_error("All bets are off for call management");
@@ -148,7 +161,7 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		if (nc && nc->status >= 2 && nc->status <= 5)
 			poll_again = TRUE;
 
-		if (oc && (!nc || (nc->id > oc->id))) {
+		if (oc && (nc == NULL || (nc->id > oc->id))) {
 			enum ofono_disconnect_reason reason;
 
 			if (vd->local_release & (0x1 << oc->id))
@@ -161,24 +174,44 @@ static void clcc_poll_cb(gboolean ok, GAtResult *result, gpointer user_data)
 								reason, NULL);
 
 			o = o->next;
-		} else if (nc && (!oc || (nc->id < oc->id))) {
+		} else if (nc && (oc == NULL || (nc->id < oc->id))) {
 			/* new call, signal it */
 			if (nc->type == 0)
 				ofono_voicecall_notify(vc, nc);
 
 			n = n->next;
 		} else {
-			/* Always use the clip_validity from old call
+			/*
+			 * Always use the clip_validity from old call
 			 * the only place this is truly told to us is
 			 * in the CLIP notify, the rest are fudged
 			 * anyway.  Useful when RING, CLIP is used,
 			 * and we're forced to use CLCC and clip_validity
 			 * is 1
 			 */
-			nc->clip_validity = oc->clip_validity;
+			if (oc->clip_validity == 1)
+				nc->clip_validity = oc->clip_validity;
 
-			if (memcmp(nc, oc, sizeof(struct ofono_call)) &&
-					!nc->type)
+			/*
+			 * CNAP doesn't arrive as part of CLCC, always
+			 * re-use from the old call
+			 */
+			strncpy(nc->name, oc->name,
+					OFONO_MAX_CALLER_NAME_LENGTH);
+			nc->name[OFONO_MAX_CALLER_NAME_LENGTH] = '\0';
+			nc->cnap_validity = oc->cnap_validity;
+
+			/*
+			 * If the CLIP is not provided and the CLIP never
+			 * arrives, or RING is used, then signal the call
+			 * here
+			 */
+			if (nc->status == 4 && (vd->flags & FLAG_NEED_CLIP)) {
+				if (nc->type == 0)
+					ofono_voicecall_notify(vc, nc);
+
+				vd->flags &= ~FLAG_NEED_CLIP;
+			} else if (memcmp(nc, oc, sizeof(*nc)) && nc->type == 0)
 				ofono_voicecall_notify(vc, nc);
 
 			n = n->next;
@@ -217,7 +250,6 @@ static void generic_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct voicecall_data *vd = ofono_voicecall_get_data(req->vc);
 	struct ofono_error error;
 
-	dump_response("generic_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (ok && req->affected_types) {
@@ -246,7 +278,6 @@ static void release_id_cb(gboolean ok, GAtResult *result,
 	struct voicecall_data *vd = ofono_voicecall_get_data(req->vc);
 	struct ofono_error error;
 
-	dump_response("release_id_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (ok)
@@ -273,8 +304,6 @@ static void atd_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	struct ofono_call *call;
 	GSList *l;
 
-	dump_response("atd_cb", ok, result);
-
 	decode_at_error(&error, g_at_result_final_response(result));
 
 	if (!ok)
@@ -302,18 +331,17 @@ static void atd_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		else
 			validity = 2;
 
-		ofono_debug("colp_notify: %s %d %d", num, type, validity);
+		DBG("colp_notify: %s %d %d", num, type, validity);
 	}
 
 	/* Generate a voice call that was just dialed, we guess the ID */
 	call = create_call(vc, 0, 0, 2, num, type, validity);
-
-	if (!call) {
+	if (call == NULL) {
 		ofono_error("Unable to malloc, call tracking will fail!");
 		return;
 	}
 
-	/* Telephonyd will generate a call with the dialed number
+	/* oFono core will generate a call with the dialed number
 	 * inside its dial callback.  Unless we got COLP information
 	 * we do not need to communicate that a call is being
 	 * dialed
@@ -338,15 +366,15 @@ static void at_dial(struct ofono_voicecall *vc,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[256];
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
 	cbd->user = vc;
 
 	if (ph->type == 145)
-		sprintf(buf, "ATD+%s", ph->number);
+		snprintf(buf, sizeof(buf), "ATD+%s", ph->number);
 	else
-		sprintf(buf, "ATD%s", ph->number);
+		snprintf(buf, sizeof(buf), "ATD%s", ph->number);
 
 	switch (clir) {
 	case OFONO_CLIR_OPTION_INVOCATION:
@@ -374,8 +402,7 @@ static void at_dial(struct ofono_voicecall *vc,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -387,7 +414,7 @@ static void at_template(const char *cmd, struct ofono_voicecall *vc,
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 	struct change_state_req *req = g_try_new0(struct change_state_req, 1);
 
-	if (!req)
+	if (req == NULL)
 		goto error;
 
 	req->vc = vc;
@@ -400,8 +427,7 @@ static void at_template(const char *cmd, struct ofono_voicecall *vc,
 		return;
 
 error:
-	if (req)
-		g_free(req);
+	g_free(req);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -415,7 +441,7 @@ static void at_answer(struct ofono_voicecall *vc,
 static void at_hangup(struct ofono_voicecall *vc,
 			ofono_voicecall_cb_t cb, void *data)
 {
-	/* Hangup all calls */
+	/* Hangup active call */
 	at_template("AT+CHUP", vc, generic_cb, 0x3f, cb, data);
 }
 
@@ -468,7 +494,7 @@ static void at_release_specific(struct ofono_voicecall *vc, int id,
 	struct release_id_req *req = g_try_new0(struct release_id_req, 1);
 	char buf[32];
 
-	if (!req)
+	if (req == NULL)
 		goto error;
 
 	req->vc = vc;
@@ -476,15 +502,14 @@ static void at_release_specific(struct ofono_voicecall *vc, int id,
 	req->data = data;
 	req->id = id;
 
-	sprintf(buf, "AT+CHLD=1%d", id);
+	snprintf(buf, sizeof(buf), "AT+CHLD=1%d", id);
 
 	if (g_at_chat_send(vd->chat, buf, none_prefix,
 				release_id_cb, req, g_free) > 0)
 		return;
 
 error:
-	if (req)
-		g_free(req);
+	g_free(req);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -494,7 +519,7 @@ static void at_private_chat(struct ofono_voicecall *vc, int id,
 {
 	char buf[32];
 
-	sprintf(buf, "AT+CHLD=2%d", id);
+	snprintf(buf, sizeof(buf), "AT+CHLD=2%d", id);
 	at_template(buf, vc, generic_cb, 0, cb, data);
 }
 
@@ -511,7 +536,7 @@ static void at_transfer(struct ofono_voicecall *vc,
 	unsigned int transfer = 0x1 | 0x2;
 
 	/* Transfer can puts held & active calls together and disconnects
-	 * from both.  However, some networks support transfering of
+	 * from both.  However, some networks support transferring of
 	 * dialing/ringing calls as well.
 	 */
 	transfer |= 0x4 | 0x8;
@@ -526,19 +551,41 @@ static void at_deflect(struct ofono_voicecall *vc,
 	char buf[128];
 	unsigned int incoming_or_waiting = (0x1 << 4) | (0x1 << 5);
 
-	sprintf(buf, "AT+CTFR=%s,%d", ph->number, ph->type);
+	snprintf(buf, sizeof(buf), "AT+CTFR=%s,%d", ph->number, ph->type);
 	at_template(buf, vc, generic_cb, incoming_or_waiting, cb, data);
 }
 
+static gboolean vts_timeout_cb(gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	struct voicecall_data *vd = cbd->user;
+	ofono_voicecall_cb_t cb = cbd->cb;
+
+	vd->vts_source = 0;
+
+	CALLBACK_WITH_SUCCESS(cb, cbd->data);
+	g_free(cbd);
+
+	return FALSE;
+}
+
 static void vts_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
+	struct voicecall_data *vd = cbd->user;
 	ofono_voicecall_cb_t cb = cbd->cb;
 	struct ofono_error error;
 
-	dump_response("vts_cb", ok, result);
 	decode_at_error(&error, g_at_result_final_response(result));
-	cb(&error, cbd->data);
+
+	if (!ok) {
+		cb(&error, cbd->data);
+
+		g_free(cbd);
+		return;
+	}
+
+	vd->vts_source = g_timeout_add(vd->vts_delay, vts_timeout_cb, cbd);
 }
 
 static void at_send_dtmf(struct ofono_voicecall *vc, const char *dtmf,
@@ -551,22 +598,25 @@ static void at_send_dtmf(struct ofono_voicecall *vc, const char *dtmf,
 	int i;
 	char *buf;
 
-	if (!cbd)
+	if (cbd == NULL)
 		goto error;
 
-	/* strlen("+VTS=\"T\";") = 9 + initial AT + null */
+	cbd->user = vd;
+
+	/* strlen("+VTS=T;") = 7 + initial AT + null */
 	buf = g_try_new(char, len * 9 + 3);
-
-	if (!buf)
+	if (buf == NULL)
 		goto error;
 
-	s = sprintf(buf, "AT+VTS=\"%c\"", dtmf[0]);
+	s = sprintf(buf, "AT+VTS=%c", dtmf[0]);
 
 	for (i = 1; i < len; i++)
-		s += sprintf(buf + s, ";+VTS=\"%c\"", dtmf[i]);
+		s += sprintf(buf + s, ";+VTS=%c", dtmf[i]);
+
+	vd->vts_delay = vd->tone_duration * len;
 
 	s = g_at_chat_send(vd->chat, buf, none_prefix,
-				vts_cb, cbd, g_free);
+				vts_cb, cbd, NULL);
 
 	g_free(buf);
 
@@ -574,8 +624,7 @@ static void at_send_dtmf(struct ofono_voicecall *vc, const char *dtmf,
 		return;
 
 error:
-	if (cbd)
-		g_free(cbd);
+	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
 }
@@ -586,8 +635,6 @@ static void ring_notify(GAtResult *result, gpointer user_data)
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 	struct ofono_call *call;
 
-	dump_response("ring_notify", TRUE, result);
-
 	/* See comment in CRING */
 	if (g_slist_find_custom(vd->calls, GINT_TO_POINTER(5),
 				at_util_call_compare_by_status))
@@ -600,14 +647,14 @@ static void ring_notify(GAtResult *result, gpointer user_data)
 
 	/* Generate an incoming call of unknown type */
 	call = create_call(vc, 9, 1, 4, NULL, 128, 2);
-
-	if (!call) {
+	if (call == NULL) {
 		ofono_error("Couldn't create call, call management is fubar!");
 		return;
 	}
 
 	/* We don't know the call type, we must run clcc */
 	vd->clcc_source = g_timeout_add(CLIP_INTERVAL, poll_clcc, vc);
+	vd->flags = FLAG_NEED_CLIP | FLAG_NEED_CNAP;
 }
 
 static void cring_notify(GAtResult *result, gpointer user_data)
@@ -617,9 +664,6 @@ static void cring_notify(GAtResult *result, gpointer user_data)
 	GAtResultIter iter;
 	const char *line;
 	int type;
-	struct ofono_call *call;
-
-	dump_response("cring_notify", TRUE, result);
 
 	/* Handle the following situation:
 	 * Active Call + Waiting Call.  Active Call is Released.  The Waiting
@@ -643,7 +687,6 @@ static void cring_notify(GAtResult *result, gpointer user_data)
 		return;
 
 	line = g_at_result_iter_raw_line(&iter);
-
 	if (line == NULL)
 		return;
 
@@ -654,7 +697,7 @@ static void cring_notify(GAtResult *result, gpointer user_data)
 		type = 9;
 
 	/* Generate an incoming call */
-	call = create_call(vc, type, 1, 4, NULL, 128, 2);
+	create_call(vc, type, 1, 4, NULL, 128, 2);
 
 	/* We have a call, and call type but don't know the number and
 	 * must wait for the CLIP to arrive before announcing the call.
@@ -662,8 +705,9 @@ static void cring_notify(GAtResult *result, gpointer user_data)
 	 * earlier, we announce the call there
 	 */
 	vd->clcc_source = g_timeout_add(CLIP_INTERVAL, poll_clcc, vc);
+	vd->flags = FLAG_NEED_CLIP | FLAG_NEED_CNAP;
 
-	ofono_debug("cring_notify");
+	DBG("");
 }
 
 static void clip_notify(GAtResult *result, gpointer user_data)
@@ -676,16 +720,17 @@ static void clip_notify(GAtResult *result, gpointer user_data)
 	GSList *l;
 	struct ofono_call *call;
 
-	dump_response("clip_notify", TRUE, result);
-
 	l = g_slist_find_custom(vd->calls, GINT_TO_POINTER(4),
 				at_util_call_compare_by_status);
-
 	if (l == NULL) {
 		ofono_error("CLIP for unknown call");
 		return;
 	}
 
+	/* We have already saw a CLIP for this call, no need to parse again */
+	if ((vd->flags & FLAG_NEED_CLIP) == 0)
+		return;
+
 	g_at_result_iter_init(&iter, result);
 
 	if (!g_at_result_iter_next(&iter, "+CLIP:"))
@@ -698,9 +743,9 @@ static void clip_notify(GAtResult *result, gpointer user_data)
 		return;
 
 	if (strlen(num) > 0)
-		validity = 0;
+		validity = CLIP_VALIDITY_VALID;
 	else
-		validity = 2;
+		validity = CLIP_VALIDITY_NOT_AVAILABLE;
 
 	/* Skip subaddr, satype and alpha */
 	g_at_result_iter_skip_next(&iter);
@@ -710,7 +755,7 @@ static void clip_notify(GAtResult *result, gpointer user_data)
 	/* If we have CLI validity field, override our guessed value */
 	g_at_result_iter_next_number(&iter, &validity);
 
-	ofono_debug("clip_notify: %s %d %d", num, type, validity);
+	DBG("%s %d %d", num, type, validity);
 
 	call = l->data;
 
@@ -723,13 +768,60 @@ static void clip_notify(GAtResult *result, gpointer user_data)
 	if (call->type == 0)
 		ofono_voicecall_notify(vc, call);
 
-	/* We started a CLCC, but the CLIP arrived and the call type
-	 * is known.  If we don't need to poll, cancel the GSource
-	 */
-	if (call->type != 9 && vd->clcc_source) {
-		g_source_remove(vd->clcc_source);
-		vd->clcc_source = 0;
+	vd->flags &= ~FLAG_NEED_CLIP;
+}
+
+static void cnap_notify(GAtResult *result, gpointer user_data)
+{
+	struct ofono_voicecall *vc = user_data;
+	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
+	GAtResultIter iter;
+	const char *name;
+	int validity;
+	GSList *l;
+	struct ofono_call *call;
+
+	l = g_slist_find_custom(vd->calls, GINT_TO_POINTER(4),
+				at_util_call_compare_by_status);
+	if (l == NULL) {
+		ofono_error("CNAP for unknown call");
+		return;
 	}
+
+	/* We have already saw a CLIP for this call, no need to parse again */
+	if ((vd->flags & FLAG_NEED_CNAP) == 0)
+		return;
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+CNAP:"))
+		return;
+
+	if (!g_at_result_iter_next_string(&iter, &name))
+		return;
+
+	if (strlen(name) > 0)
+		validity = CNAP_VALIDITY_VALID;
+	else
+		validity = CNAP_VALIDITY_NOT_AVAILABLE;
+
+	/* If we have CNI validity field, override our guessed value */
+	g_at_result_iter_next_number(&iter, &validity);
+
+	DBG("%s %d", name, validity);
+
+	call = l->data;
+
+	strncpy(call->name, name,
+		OFONO_MAX_CALLER_NAME_LENGTH);
+	call->name[OFONO_MAX_CALLER_NAME_LENGTH] = '\0';
+	call->cnap_validity = validity;
+
+	/* Only signal the call here if we already signaled it to the core */
+	if (call->type == 0 && (vd->flags & FLAG_NEED_CLIP) == 0)
+		ofono_voicecall_notify(vc, call);
+
+	vd->flags &= ~FLAG_NEED_CNAP;
 }
 
 static void ccwa_notify(GAtResult *result, gpointer user_data)
@@ -741,8 +833,6 @@ static void ccwa_notify(GAtResult *result, gpointer user_data)
 	int num_type, validity, cls;
 	struct ofono_call *call;
 
-	dump_response("ccwa_notify", TRUE, result);
-
 	/* Some modems resend CCWA, ignore it the second time around */
 	if (g_slist_find_custom(vd->calls, GINT_TO_POINTER(5),
 				at_util_call_compare_by_status))
@@ -773,12 +863,11 @@ static void ccwa_notify(GAtResult *result, gpointer user_data)
 	/* If we have CLI validity field, override our guessed value */
 	g_at_result_iter_next_number(&iter, &validity);
 
-	ofono_debug("ccwa_notify: %s %d %d %d", num, num_type, cls, validity);
+	DBG("%s %d %d %d", num, num_type, cls, validity);
 
 	call = create_call(vc, class_to_call_type(cls), 1, 5,
 				num, num_type, validity);
-
-	if (!call) {
+	if (call == NULL) {
 		ofono_error("Unable to malloc. Call management is fubar");
 		return;
 	}
@@ -822,17 +911,38 @@ static void busy_notify(GAtResult *result, gpointer user_data)
 			clcc_poll_cb, vc, NULL);
 }
 
+static void vtd_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct ofono_voicecall *vc = user_data;
+	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
+	GAtResultIter iter;
+	int duration;
+
+	if (!ok)
+		return;
+
+	g_at_result_iter_init(&iter, result);
+	g_at_result_iter_next(&iter, "+VTD:");
+
+	if (!g_at_result_iter_next_number(&iter, &duration))
+		return;
+
+	if (duration)
+		vd->tone_duration = duration * 100;
+}
+
 static void at_voicecall_initialized(gboolean ok, GAtResult *result,
 					gpointer user_data)
 {
 	struct ofono_voicecall *vc = user_data;
 	struct voicecall_data *vd = ofono_voicecall_get_data(vc);
 
-	ofono_debug("voicecall_init: registering to notifications");
+	DBG("voicecall_init: registering to notifications");
 
 	g_at_chat_register(vd->chat, "RING", ring_notify, FALSE, vc, NULL);
 	g_at_chat_register(vd->chat, "+CRING:", cring_notify, FALSE, vc, NULL);
 	g_at_chat_register(vd->chat, "+CLIP:", clip_notify, FALSE, vc, NULL);
+	g_at_chat_register(vd->chat, "+CNAP:", cnap_notify, FALSE, vc, NULL);
 	g_at_chat_register(vd->chat, "+CCWA:", ccwa_notify, FALSE, vc, NULL);
 
 	/* Modems with 'better' call progress indicators should
@@ -856,16 +966,25 @@ static int at_voicecall_probe(struct ofono_voicecall *vc, unsigned int vendor,
 	GAtChat *chat = data;
 	struct voicecall_data *vd;
 
-	vd = g_new0(struct voicecall_data, 1);
-	vd->chat = chat;
+	vd = g_try_new0(struct voicecall_data, 1);
+	if (vd == NULL)
+		return -ENOMEM;
+
+	vd->chat = g_at_chat_clone(chat);
+	vd->vendor = vendor;
+	vd->tone_duration = TONE_DURATION;
 
 	ofono_voicecall_set_data(vc, vd);
 
-	g_at_chat_send(chat, "AT+CRC=1", NULL, NULL, NULL, NULL);
-	g_at_chat_send(chat, "AT+CLIP=1", NULL, NULL, NULL, NULL);
-	g_at_chat_send(chat, "AT+COLP=1", NULL, NULL, NULL, NULL);
-	g_at_chat_send(chat, "AT+CCWA=1", NULL,
+	g_at_chat_send(vd->chat, "AT+CRC=1", NULL, NULL, NULL, NULL);
+	g_at_chat_send(vd->chat, "AT+CLIP=1", NULL, NULL, NULL, NULL);
+	g_at_chat_send(vd->chat, "AT+CNAP=1", NULL, NULL, NULL, NULL);
+	g_at_chat_send(vd->chat, "AT+COLP=1", NULL, NULL, NULL, NULL);
+	g_at_chat_send(vd->chat, "AT+VTD?", NULL,
+				vtd_query_cb, vc, NULL);
+	g_at_chat_send(vd->chat, "AT+CCWA=1", NULL,
 				at_voicecall_initialized, vc, NULL);
+
 	return 0;
 }
 
@@ -876,11 +995,15 @@ static void at_voicecall_remove(struct ofono_voicecall *vc)
 	if (vd->clcc_source)
 		g_source_remove(vd->clcc_source);
 
+	if (vd->vts_source)
+		g_source_remove(vd->vts_source);
+
 	g_slist_foreach(vd->calls, (GFunc) g_free, NULL);
 	g_slist_free(vd->calls);
 
 	ofono_voicecall_set_data(vc, NULL);
 
+	g_at_chat_unref(vd->chat);
 	g_free(vd);
 }
 
@@ -890,7 +1013,7 @@ static struct ofono_voicecall_driver driver = {
 	.remove			= at_voicecall_remove,
 	.dial			= at_dial,
 	.answer			= at_answer,
-	.hangup			= at_hangup,
+	.hangup_all		= at_hangup,
 	.hold_all_active	= at_hold_all_active,
 	.release_all_held	= at_release_all_held,
 	.set_udub		= at_set_udub,
