@@ -128,7 +128,7 @@ static void hso_set_rat_mode(struct ofono_radio_settings *rs,
 	struct radio_settings_data *rsd = ofono_radio_settings_get_data(rs);
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[20];
-	int value;
+	int value = 5;
 
 	switch (mode) {
 	case OFONO_RADIO_ACCESS_MODE_ANY:
@@ -140,19 +140,19 @@ static void hso_set_rat_mode(struct ofono_radio_settings *rs,
 	case OFONO_RADIO_ACCESS_MODE_UMTS:
 		value = 1;
 		break;
-	default:
-		CALLBACK_WITH_FAILURE(cb, data);
-		g_free(cbd);
-		return;
+	case OFONO_RADIO_ACCESS_MODE_LTE:
+		goto error;
 	}
 
 	snprintf(buf, sizeof(buf), "AT_OPSYS=%u,2", value);
 
 	if (g_at_chat_send(rsd->chat, buf, none_prefix,
-					opsys_modify_cb, cbd, g_free) == 0) {
-		CALLBACK_WITH_FAILURE(cb, data);
-		g_free(cbd);
-	}
+					opsys_modify_cb, cbd, g_free) > 0)
+		return;
+
+error:
+	CALLBACK_WITH_FAILURE(cb, data);
+	g_free(cbd);
 }
 
 static void opsys_support_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -172,13 +172,15 @@ static int hso_radio_settings_probe(struct ofono_radio_settings *rs,
 	struct radio_settings_data *rsd;
 
 	rsd = g_try_new0(struct radio_settings_data, 1);
-	if (!rsd)
+	if (rsd == NULL)
 		return -ENOMEM;
 
 	rsd->chat = g_at_chat_clone(chat);
 
 	ofono_radio_settings_set_data(rs, rsd);
 
+	g_at_chat_send(rsd->chat, "AT_OPBM?", none_prefix, NULL, NULL, NULL);
+
 	g_at_chat_send(rsd->chat, "AT_OPSYS=?", opsys_prefix,
 					opsys_support_cb, rs, NULL);
 
