@@ -60,6 +60,7 @@ enum ofono_error_type {
 	OFONO_ERROR_TYPE_CME,
 	OFONO_ERROR_TYPE_CMS,
 	OFONO_ERROR_TYPE_CEER,
+	OFONO_ERROR_TYPE_SIM,
 	OFONO_ERROR_TYPE_FAILURE
 };
 
@@ -76,12 +77,21 @@ struct ofono_error {
 };
 
 #define OFONO_MAX_PHONE_NUMBER_LENGTH 20
+#define OFONO_MAX_CALLER_NAME_LENGTH 80
 
 struct ofono_phone_number {
 	char number[OFONO_MAX_PHONE_NUMBER_LENGTH + 1];
 	int type;
 };
 
+/* Length of NUM_FIELDS in 3GPP2 C.S0005-E v2.0 */
+#define OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH 256
+
+struct ofono_cdma_phone_number {
+	/* char maps to max size of CHARi (8 bit) in 3GPP2 C.S0005-E v2.0 */
+	char number[OFONO_CDMA_MAX_PHONE_NUMBER_LENGTH];
+};
+
 struct ofono_call {
 	unsigned int id;
 	int type;
@@ -89,7 +99,9 @@ struct ofono_call {
 	int status;
 	ofono_bool_t mpty;
 	struct ofono_phone_number phone_number;
+	char name[OFONO_MAX_CALLER_NAME_LENGTH + 1];
 	int clip_validity;
+	int cnap_validity;
 };
 
 struct ofono_network_time {
@@ -103,6 +115,14 @@ struct ofono_network_time {
 	int utcoff;	/* Offset from UTC in seconds */
 };
 
+#define OFONO_SHA1_UUID_LEN 20
+
+struct ofono_uuid {
+	unsigned char uuid[OFONO_SHA1_UUID_LEN];
+};
+
+const char *ofono_uuid_to_str(const struct ofono_uuid *uuid);
+
 #ifdef __cplusplus
 }
 #endif
