@@ -34,8 +34,10 @@
 #include "ofono.h"
 
 #include "common.h"
+#include "simutil.h"
 
 #define CALL_FORWARDING_FLAG_CACHED 0x1
+#define CALL_FORWARDING_FLAG_CPHS_CFF 0x2
 
 /* According to 27.007 Spec */
 #define DEFAULT_NO_REPLY_TIMEOUT 20
@@ -43,12 +45,12 @@
 static GSList *g_drivers = NULL;
 
 enum call_forwarding_type {
-	CALL_FORWARDING_TYPE_UNCONDITIONAL = 0,
-	CALL_FORWARDING_TYPE_BUSY = 1,
-	CALL_FORWARDING_TYPE_NO_REPLY = 2,
-	CALL_FORWARDING_TYPE_NOT_REACHABLE = 3,
-	CALL_FORWARDING_TYPE_ALL = 4,
-	CALL_FORWARDING_TYPE_ALL_CONDITIONAL = 5
+	CALL_FORWARDING_TYPE_UNCONDITIONAL =		0,
+	CALL_FORWARDING_TYPE_BUSY =			1,
+	CALL_FORWARDING_TYPE_NO_REPLY =			2,
+	CALL_FORWARDING_TYPE_NOT_REACHABLE =		3,
+	CALL_FORWARDING_TYPE_ALL =			4,
+	CALL_FORWARDING_TYPE_ALL_CONDITIONAL =		5
 };
 
 struct ofono_call_forwarding {
@@ -58,6 +60,8 @@ struct ofono_call_forwarding {
 	int query_next;
 	int query_end;
 	struct cf_ss_request *ss_req;
+	struct ofono_sim *sim;
+	unsigned char cfis_record_id;
 	struct ofono_ussd *ussd;
 	unsigned int ussd_watch;
 	const struct ofono_call_forwarding_driver *driver;
@@ -113,7 +117,7 @@ static int cf_find_timeout(GSList *cf_list, int cls)
 	l = g_slist_find_custom(cf_list, GINT_TO_POINTER(cls),
 		cf_condition_find_with_cls);
 
-	if (!l)
+	if (l == NULL)
 		return DEFAULT_NO_REPLY_TIMEOUT;
 
 	c = l->data;
@@ -158,7 +162,7 @@ static GSList *cf_cond_list_create(int total,
 				continue;
 
 			cond = g_try_new0(struct ofono_call_forwarding_condition, 1);
-			if (!cond)
+			if (cond == NULL)
 				continue;
 
 			memcpy(cond, &list[i],
@@ -202,6 +206,94 @@ static const char *cf_type_lut[] = {
 	"AllConditional"
 };
 
+static void sim_cfis_update_cb(int ok, void *data)
+{
+	if (!ok)
+		ofono_info("Failed to update EFcfis");
+}
+
+static void sim_cphs_cff_update_cb(int ok, void *data)
+{
+	if (!ok)
+		ofono_info("Failed to update EFcphs-cff");
+}
+
+static gboolean is_cfu_enabled(struct ofono_call_forwarding *cf,
+				struct ofono_call_forwarding_condition **out)
+{
+	GSList *l = cf->cf_conditions[CALL_FORWARDING_TYPE_UNCONDITIONAL];
+	struct ofono_call_forwarding_condition *cond;
+
+	/*
+	 * For now we only support Voice, although Fax & all Data
+	 * basic services are applicable as well.
+	 */
+	for (; l; l = l->next) {
+		cond = l->data;
+
+		if (cond->cls > BEARER_CLASS_VOICE)
+			continue;
+
+		if (out)
+			*out = cond;
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static void sim_set_cf_indicator(struct ofono_call_forwarding *cf)
+{
+	gboolean cfu_voice;
+	struct ofono_call_forwarding_condition *cond;
+
+	cfu_voice = is_cfu_enabled(cf, &cond);
+
+	if (cf->cfis_record_id) {
+		unsigned char data[16];
+		int number_len;
+
+		memset(data, 0xff, sizeof(data));
+
+		/* Profile Identifier */
+		data[0] = 0x01;
+
+		if (cfu_voice) {
+			number_len = strlen(cond->phone_number.number);
+
+			/* CFU indicator Status - Voice */
+			data[1] = 0x01;
+			number_len = (number_len + 1) / 2;
+			data[2] = number_len + 1;
+			data[3] = cond->phone_number.type;
+
+			sim_encode_bcd_number(cond->phone_number.number,
+						data + 4);
+		} else {
+			data[1] = 0x00;
+			data[2] = 1;
+			data[3] = 128;
+		}
+
+		ofono_sim_write(cf->sim, SIM_EFCFIS_FILEID,
+					sim_cfis_update_cb,
+					OFONO_SIM_FILE_STRUCTURE_FIXED,
+					cf->cfis_record_id, data,
+					sizeof(data), cf);
+		return;
+	}
+
+	if (cf->flags & CALL_FORWARDING_FLAG_CPHS_CFF) {
+		unsigned char cff_voice = cfu_voice ? 0x0A : 0x05;
+
+		ofono_sim_write(cf->sim, SIM_EF_CPHS_CFF_FILEID,
+					sim_cphs_cff_update_cb,
+					OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+					0, &cff_voice, sizeof(cff_voice), cf);
+	}
+}
+
 static void set_new_cond_list(struct ofono_call_forwarding *cf,
 				int type, GSList *list)
 {
@@ -216,6 +308,15 @@ static void set_new_cond_list(struct ofono_call_forwarding *cf,
 	dbus_uint16_t timeout;
 	char attr[64];
 	char tattr[64];
+	gboolean update_sim = FALSE;
+	gboolean old_cfu;
+	gboolean new_cfu;
+
+	if ((cf->flags & CALL_FORWARDING_FLAG_CPHS_CFF) ||
+			cf->cfis_record_id > 0)
+		old_cfu = is_cfu_enabled(cf, NULL);
+	else
+		old_cfu = FALSE;
 
 	for (l = list; l; l = l->next) {
 		lc = l->data;
@@ -246,12 +347,16 @@ static void set_new_cond_list(struct ofono_call_forwarding *cf,
 
 			if (oc->phone_number.type != lc->phone_number.type ||
 				strcmp(oc->phone_number.number,
-					lc->phone_number.number))
+					lc->phone_number.number)) {
 				ofono_dbus_signal_property_changed(conn, path,
 						OFONO_CALL_FORWARDING_INTERFACE,
 						attr, DBUS_TYPE_STRING,
 						&number);
 
+				if (type == CALL_FORWARDING_TYPE_UNCONDITIONAL)
+					update_sim = TRUE;
+			}
+
 			if (type == CALL_FORWARDING_TYPE_NO_REPLY &&
 				oc->time != lc->time)
 				ofono_dbus_signal_property_changed(conn, path,
@@ -270,6 +375,9 @@ static void set_new_cond_list(struct ofono_call_forwarding *cf,
 						attr, DBUS_TYPE_STRING,
 						&number);
 
+			if (type == CALL_FORWARDING_TYPE_UNCONDITIONAL)
+				update_sim = TRUE;
+
 			if (type == CALL_FORWARDING_TYPE_NO_REPLY &&
 				lc->time != DEFAULT_NO_REPLY_TIMEOUT)
 				ofono_dbus_signal_property_changed(conn, path,
@@ -285,6 +393,13 @@ static void set_new_cond_list(struct ofono_call_forwarding *cf,
 	for (o = old; o; o = o->next) {
 		oc = o->data;
 
+		/*
+		 * For now we only support Voice, although Fax & all Data
+		 * basic services are applicable as well.
+		 */
+		if (oc->cls > BEARER_CLASS_VOICE)
+			continue;
+
 		snprintf(attr, sizeof(attr), "%s%s",
 			bearer_class_to_string(oc->cls), cf_type_lut[type]);
 
@@ -295,6 +410,9 @@ static void set_new_cond_list(struct ofono_call_forwarding *cf,
 					OFONO_CALL_FORWARDING_INTERFACE, attr,
 					DBUS_TYPE_STRING, &number);
 
+		if (type == CALL_FORWARDING_TYPE_UNCONDITIONAL)
+			update_sim = TRUE;
+
 		if (type == CALL_FORWARDING_TYPE_NO_REPLY &&
 			oc->time != DEFAULT_NO_REPLY_TIMEOUT)
 			ofono_dbus_signal_property_changed(conn, path,
@@ -305,6 +423,24 @@ static void set_new_cond_list(struct ofono_call_forwarding *cf,
 
 	cf_list_clear(old);
 	cf->cf_conditions[type] = list;
+
+	if (update_sim == TRUE)
+		sim_set_cf_indicator(cf);
+
+	if ((cf->flags & CALL_FORWARDING_FLAG_CPHS_CFF) ||
+			cf->cfis_record_id > 0)
+		new_cfu = is_cfu_enabled(cf, NULL);
+	else
+		new_cfu = FALSE;
+
+	if (new_cfu != old_cfu) {
+		ofono_bool_t status = new_cfu;
+
+		ofono_dbus_signal_property_changed(conn, path,
+					OFONO_CALL_FORWARDING_INTERFACE,
+					"ForwardingFlagOnSim",
+					DBUS_TYPE_BOOLEAN, &status);
+	}
 }
 
 static inline void property_append_cf_condition(DBusMessageIter *dict, int cls,
@@ -345,7 +481,7 @@ static void property_append_cf_conditions(DBusMessageIter *dict,
 		while (l && (cf = l->data) && (cf->cls < i))
 				l = l->next;
 
-		if (!l || cf->cls != i) {
+		if (l == NULL || cf->cls != i) {
 			property_append_cf_condition(dict, i, postfix, "",
 						DEFAULT_NO_REPLY_TIMEOUT);
 			continue;
@@ -365,10 +501,10 @@ static DBusMessage *cf_get_properties_reply(DBusMessage *msg,
 	DBusMessageIter iter;
 	DBusMessageIter dict;
 	int i;
+	dbus_bool_t status;
 
 	reply = dbus_message_new_method_return(msg);
-
-	if (!reply)
+	if (reply == NULL)
 		return NULL;
 
 	dbus_message_iter_init_append(reply, &iter);
@@ -382,6 +518,15 @@ static DBusMessage *cf_get_properties_reply(DBusMessage *msg,
 						BEARER_CLASS_VOICE,
 						cf_type_lut[i]);
 
+	if ((cf->flags & CALL_FORWARDING_FLAG_CPHS_CFF) ||
+			cf->cfis_record_id > 0)
+		status = is_cfu_enabled(cf, NULL);
+	else
+		status = FALSE;
+
+	ofono_dbus_dict_append(&dict, "ForwardingFlagOnSim", DBUS_TYPE_BOOLEAN,
+					&status);
+
 	dbus_message_iter_close_container(&iter, &dict);
 
 	return reply;
@@ -425,11 +570,13 @@ static DBusMessage *cf_get_properties(DBusConnection *conn, DBusMessage *msg,
 					void *data)
 {
 	struct ofono_call_forwarding *cf = data;
+	struct ofono_modem *modem = __ofono_atom_get_modem(cf->atom);
 
-	if (cf->flags & CALL_FORWARDING_FLAG_CACHED)
+	if ((cf->flags & CALL_FORWARDING_FLAG_CACHED) ||
+			ofono_modem_get_online(modem) == FALSE)
 		return cf_get_properties_reply(msg, cf);
 
-	if (!cf->driver->query)
+	if (cf->driver->query == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (__ofono_call_forwarding_is_busy(cf) ||
@@ -584,12 +731,16 @@ static DBusMessage *cf_set_property(DBusConnection *conn, DBusMessage *msg,
 					void *data)
 {
 	struct ofono_call_forwarding *cf = data;
+	struct ofono_modem *modem = __ofono_atom_get_modem(cf->atom);
 	DBusMessageIter iter;
 	DBusMessageIter var;
 	const char *property;
 	int cls;
 	int type;
 
+	if (ofono_modem_get_online(modem) == FALSE)
+		return __ofono_error_not_available(msg);
+
 	if (__ofono_call_forwarding_is_busy(cf) ||
 			__ofono_ussd_is_busy(cf->ussd))
 		return __ofono_error_busy(msg);
@@ -627,7 +778,7 @@ static DBusMessage *cf_set_property(DBusConnection *conn, DBusMessage *msg,
 				GINT_TO_POINTER(cls),
 				cf_condition_find_with_cls);
 
-		if (!l)
+		if (l == NULL)
 			return __ofono_error_failed(msg);
 
 		c = l->data;
@@ -706,7 +857,7 @@ static DBusMessage *cf_disable_all(DBusConnection *conn, DBusMessage *msg,
 	const char *strtype;
 	int type;
 
-	if (!cf->driver->erasure)
+	if (cf->driver->erasure == NULL)
 		return __ofono_error_not_implemented(msg);
 
 	if (__ofono_call_forwarding_is_busy(cf) ||
@@ -898,7 +1049,7 @@ static gboolean cf_ss_control(int type, const char *sc,
 	void *operation = NULL;
 
 	/* Before we do anything, make sure we're actually initialized */
-	if (!cf)
+	if (cf == NULL)
 		return FALSE;
 
 	if (__ofono_call_forwarding_is_busy(cf)) {
@@ -1000,7 +1151,7 @@ static gboolean cf_ss_control(int type, const char *sc,
 		break;
 	}
 
-	if (!operation) {
+	if (operation == NULL) {
 		reply = __ofono_error_not_implemented(msg);
 		g_dbus_send_message(conn, reply);
 
@@ -1009,7 +1160,7 @@ static gboolean cf_ss_control(int type, const char *sc,
 
 	cf->ss_req = g_try_new0(struct cf_ss_request, 1);
 
-	if (!cf->ss_req) {
+	if (cf->ss_req == NULL) {
 		reply = __ofono_error_failed(msg);
 		g_dbus_send_message(conn, reply);
 
@@ -1105,6 +1256,122 @@ gboolean __ofono_call_forwarding_is_busy(struct ofono_call_forwarding *cf)
 	return cf->pending ? TRUE : FALSE;
 }
 
+static void sim_cfis_read_cb(int ok, int total_length, int record,
+			const unsigned char *data,
+			int record_length, void *userdata)
+{
+	struct ofono_call_forwarding *cf = userdata;
+	DBusConnection *conn = ofono_dbus_get_connection();
+	const char *path = __ofono_atom_get_path(cf->atom);
+
+	if (!ok || record_length < 16 || total_length < record_length) {
+		cf->cfis_record_id = 0;
+		return;
+	}
+
+	/*
+	 * Multiple Subscriber Profile number which can have values 1-4.
+	 * Profile id 1 is assumed as the current profile.
+	 */
+	if (data[0] != 1)
+		return;
+
+	cf->cfis_record_id = record;
+
+	/*
+	 * For now we only support Voice, although Fax & all Data
+	 * basic services are applicable as well.
+	 */
+	if (data[1] & 0x01) {
+		int ton_npi;
+		int number_len;
+		const char *number;
+		char attr[64];
+		struct ofono_call_forwarding_condition *cond;
+		dbus_bool_t status;
+
+		number_len = data[2];
+		ton_npi = data[3];
+
+		if (number_len > 11 || ton_npi == 0xff)
+			return;
+
+		cond = g_try_new0(struct ofono_call_forwarding_condition, 1);
+		if (cond == NULL)
+			return;
+
+		status = TRUE;
+		cond->status = TRUE;
+		cond->cls = BEARER_CLASS_VOICE;
+		cond->time = 0;
+		cond->phone_number.type = ton_npi;
+
+		sim_extract_bcd_number(data + 4, number_len - 1,
+					cond->phone_number.number);
+		number = phone_number_to_string(&cond->phone_number);
+
+		snprintf(attr, sizeof(attr), "%s%s",
+			bearer_class_to_string(BEARER_CLASS_VOICE),
+			cf_type_lut[CALL_FORWARDING_TYPE_UNCONDITIONAL]);
+
+		cf->cf_conditions[CALL_FORWARDING_TYPE_UNCONDITIONAL] =
+						g_slist_append(NULL, cond);
+
+		ofono_dbus_signal_property_changed(conn, path,
+					OFONO_CALL_FORWARDING_INTERFACE,
+					attr, DBUS_TYPE_STRING, &number);
+
+		ofono_dbus_signal_property_changed(conn, path,
+					OFONO_CALL_FORWARDING_INTERFACE,
+					"ForwardingFlagOnSim",
+					DBUS_TYPE_BOOLEAN, &status);
+	}
+}
+
+static void sim_cphs_cff_read_cb(int ok, int total_length, int record,
+				const unsigned char *data,
+				int record_length, void *userdata)
+{
+	struct ofono_call_forwarding *cf = userdata;
+	DBusConnection *conn = ofono_dbus_get_connection();
+	const char *path = __ofono_atom_get_path(cf->atom);
+	dbus_bool_t cfu_voice;
+
+	if (!ok || total_length < 1) {
+		return;
+	}
+
+	cf->flags |= CALL_FORWARDING_FLAG_CPHS_CFF;
+
+	/*
+	 * For now we only support Voice, although Fax & all Data
+	 * basic services are applicable as well.
+	 */
+	if ((data[0] & 0xf) != 0xA)
+		return;
+
+	cfu_voice = TRUE;
+
+	ofono_dbus_signal_property_changed(conn, path,
+					OFONO_CALL_FORWARDING_INTERFACE,
+					"ForwardingFlagOnSim",
+					DBUS_TYPE_BOOLEAN, &cfu_voice);
+}
+
+static void sim_read_cf_indicator(struct ofono_call_forwarding *cf)
+{
+	if (__ofono_sim_service_available(cf->sim,
+			SIM_UST_SERVICE_CFIS,
+			SIM_SST_SERVICE_CFIS) == TRUE)
+		ofono_sim_read(cf->sim, SIM_EFCFIS_FILEID,
+				OFONO_SIM_FILE_STRUCTURE_FIXED,
+				sim_cfis_read_cb, cf);
+	else
+		ofono_sim_read(cf->sim, SIM_EF_CPHS_CFF_FILEID,
+				OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+				sim_cphs_cff_read_cb, cf);
+}
+
 int ofono_call_forwarding_driver_register(const struct ofono_call_forwarding_driver *d)
 {
 	DBG("driver: %p, name: %s", d, d->name);
@@ -1213,6 +1480,7 @@ void ofono_call_forwarding_register(struct ofono_call_forwarding *cf)
 	DBusConnection *conn = ofono_dbus_get_connection();
 	const char *path = __ofono_atom_get_path(cf->atom);
 	struct ofono_modem *modem = __ofono_atom_get_modem(cf->atom);
+	struct ofono_atom *sim_atom;
 	struct ofono_atom *ussd_atom;
 
 	if (!g_dbus_register_interface(conn, path,
@@ -1227,6 +1495,14 @@ void ofono_call_forwarding_register(struct ofono_call_forwarding *cf)
 
 	ofono_modem_add_interface(modem, OFONO_CALL_FORWARDING_INTERFACE);
 
+	sim_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_SIM);
+
+	if (sim_atom) {
+		cf->sim = __ofono_atom_get_data(sim_atom);
+
+		sim_read_cf_indicator(cf);
+	}
+
 	cf->ussd_watch = __ofono_modem_add_atom_watch(modem,
 					OFONO_ATOM_TYPE_USSD,
 					ussd_watch, cf, NULL);
