@@ -28,7 +28,7 @@
 
 #include "ofono.h"
 
-#define DBUS_GSM_ERROR_INTERFACE "org.ofono.Error"
+#define OFONO_ERROR_INTERFACE "org.ofono.Error"
 
 static DBusConnection *g_connection;
 
@@ -75,7 +75,7 @@ static void append_array_variant(DBusMessageIter *iter, int type, void *val)
 	DBusMessageIter variant, array;
 	char typesig[2];
 	char arraysig[3];
-	const char **str_array = *(const char ***)val;
+	const char **str_array = *(const char ***) val;
 	int i;
 
 	arraysig[0] = DBUS_TYPE_ARRAY;
@@ -117,7 +117,7 @@ static void append_dict_variant(DBusMessageIter *iter, int type, void *val)
 	DBusMessageIter variant, array, entry;
 	char typesig[5];
 	char arraysig[6];
-	const void **val_array = *(const void ***)val;
+	const void **val_array = *(const void ***) val;
 	int i;
 
 	arraysig[0] = DBUS_TYPE_ARRAY;
@@ -139,8 +139,24 @@ static void append_dict_variant(DBusMessageIter *iter, int type, void *val)
 
 		dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING,
 						&(val_array[i + 0]));
-		dbus_message_iter_append_basic(&entry, type,
-						&(val_array[i + 1]));
+
+		/*
+		 * D-Bus expects a char** or uint8* depending on the type
+		 * given. Since we are dealing with an array through a void**
+		 * (and thus val_array[i] is a pointer) we need to
+		 * differentiate DBUS_TYPE_STRING from the others. The other
+		 * option would be the user to pass the exact type to this
+		 * function, instead of a pointer to it. However in this case
+		 * a cast from type to void* would be needed, which is not
+		 * good.
+		 */
+		if (type == DBUS_TYPE_STRING) {
+			dbus_message_iter_append_basic(&entry, type,
+							&(val_array[i + 1]));
+		} else {
+			dbus_message_iter_append_basic(&entry, type,
+							val_array[i + 1]);
+		}
 
 		dbus_message_iter_close_container(&array, &entry);
 	}
@@ -175,8 +191,7 @@ int ofono_dbus_signal_property_changed(DBusConnection *conn,
 	DBusMessageIter iter;
 
 	signal = dbus_message_new_signal(path, interface, "PropertyChanged");
-
-	if (!signal) {
+	if (signal == NULL) {
 		ofono_error("Unable to allocate new %s.PropertyChanged signal",
 				interface);
 		return -1;
@@ -202,8 +217,7 @@ int ofono_dbus_signal_array_property_changed(DBusConnection *conn,
 	DBusMessageIter iter;
 
 	signal = dbus_message_new_signal(path, interface, "PropertyChanged");
-
-	if (!signal) {
+	if (signal == NULL) {
 		ofono_error("Unable to allocate new %s.PropertyChanged signal",
 				interface);
 		return -1;
@@ -229,8 +243,7 @@ int ofono_dbus_signal_dict_property_changed(DBusConnection *conn,
 	DBusMessageIter iter;
 
 	signal = dbus_message_new_signal(path, interface, "PropertyChanged");
-
-	if (!signal) {
+	if (signal == NULL) {
 		ofono_error("Unable to allocate new %s.PropertyChanged signal",
 				interface);
 		return -1;
@@ -247,52 +260,52 @@ int ofono_dbus_signal_dict_property_changed(DBusConnection *conn,
 
 DBusMessage *__ofono_error_invalid_args(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE
 					".InvalidArguments",
 					"Invalid arguments in method call");
 }
 
 DBusMessage *__ofono_error_invalid_format(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE
 					".InvalidFormat",
 					"Argument format is not recognized");
 }
 
 DBusMessage *__ofono_error_not_implemented(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE
 					".NotImplemented",
 					"Implementation not provided");
 }
 
 DBusMessage *__ofono_error_failed(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE ".Failed",
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".Failed",
 					"Operation failed");
 }
 
 DBusMessage *__ofono_error_busy(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE ".InProgress",
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".InProgress",
 					"Operation already in progress");
 }
 
 DBusMessage *__ofono_error_not_found(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE ".NotFound",
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".NotFound",
 			"Object is not found or not valid for this operation");
 }
 
 DBusMessage *__ofono_error_not_active(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE ".NotActive",
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".NotActive",
 			"Operation is not active or in progress");
 }
 
 DBusMessage *__ofono_error_not_supported(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE
 					".NotSupported",
 					"Operation is not supported by the"
 					" network / modem");
@@ -300,48 +313,54 @@ DBusMessage *__ofono_error_not_supported(DBusMessage *msg)
 
 DBusMessage *__ofono_error_not_available(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE
 					".NotAvailable",
 					"Operation currently not available");
 }
 
 DBusMessage *__ofono_error_timed_out(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE ".Timedout",
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".Timedout",
 			"Operation failure due to timeout");
 }
 
 DBusMessage *__ofono_error_sim_not_ready(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE ".SimNotReady",
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".SimNotReady",
 			"SIM is not ready or not inserted");
 }
 
 DBusMessage *__ofono_error_in_use(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE ".InUse",
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".InUse",
 			"The resource is currently in use");
 }
 
 DBusMessage *__ofono_error_not_attached(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE ".NotAttached",
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".NotAttached",
 			"GPRS is not attached");
 }
 
 DBusMessage *__ofono_error_attach_in_progress(DBusMessage *msg)
 {
 	return g_dbus_create_error(msg,
-				DBUS_GSM_ERROR_INTERFACE ".AttachInProgress",
+				OFONO_ERROR_INTERFACE ".AttachInProgress",
 				"GPRS Attach is in progress");
 }
 
 DBusMessage *__ofono_error_canceled(DBusMessage *msg)
 {
-	return g_dbus_create_error(msg, DBUS_GSM_ERROR_INTERFACE ".Canceled",
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".Canceled",
 					"Operation has been canceled");
 }
 
+DBusMessage *__ofono_error_access_denied(DBusMessage *msg)
+{
+	return g_dbus_create_error(msg, OFONO_ERROR_INTERFACE ".AccessDenied",
+					"Operation not permitted");
+}
+
 void __ofono_dbus_pending_reply(DBusMessage **msg, DBusMessage *reply)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
@@ -420,7 +439,7 @@ void __ofono_dbus_cleanup(void)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
 
-	if (!conn || !dbus_connection_get_is_connected(conn))
+	if (conn == NULL || !dbus_connection_get_is_connected(conn))
 		return;
 
 	dbus_gsm_set_connection(NULL);
