@@ -69,6 +69,7 @@ static struct sim_ef_info ef_db[] = {
 {	0x2F05, ROOTMF, BINARY, 0,	ALW,	PIN	},
 {	0x2F06, ROOTMF, RECORD, 0,	ALW,	PIN	},
 {	0x2FE2, ROOTMF, BINARY, 10,	ALW,	NEV 	},
+{	0x4F20, 0x5F50, BINARY, 0,	PIN,	ADM	},
 {	0x6F05, 0x7F20, BINARY, 0,	ALW,	PIN	},
 {	0x6F06, 0x0000, RECORD, 0,	ALW,	ADM	},
 {	0x6F2C, 0x7F20, BINARY, 16,	PIN,	PIN	},
@@ -812,7 +813,7 @@ static char *sim_network_name_parse(const unsigned char *buffer, int length,
 			if (buffer[i] == 0xff && buffer[i + 1] == 0xff)
 				break;
 
-		ret = g_convert((const char *)buffer, length,
+		ret = g_convert((const char *) buffer, length,
 					"UTF-8//TRANSLIT", "UCS-2BE",
 					NULL, NULL, NULL);
 		break;
@@ -938,7 +939,7 @@ struct sim_spdi *sim_spdi_new(const guint8 *tlv, int length)
 	plmn_list = ber_tlv_find_by_tag(plmn_list_tlv, 0x80, tlv_length,
 						&list_length);
 
-	if (!plmn_list)
+	if (plmn_list == NULL)
 		return NULL;
 
 	spdi = g_new0(struct sim_spdi, 1);
@@ -962,7 +963,7 @@ gboolean sim_spdi_lookup(struct sim_spdi *spdi,
 {
 	struct spdi_operator spdi_op;
 
-	if (!spdi)
+	if (spdi == NULL)
 		return FALSE;
 
 	g_strlcpy(spdi_op.mcc, mcc, sizeof(spdi_op.mcc));
@@ -1010,7 +1011,7 @@ void sim_eons_add_pnn_record(struct sim_eons *eons, int record,
 
 	name = ber_tlv_find_by_tag(tlv, 0x43, length, &namelength);
 
-	if (!name || !namelength)
+	if (name == NULL || !namelength)
 		return;
 
 	oper->longname = sim_network_name_parse(name, namelength,
@@ -1118,7 +1119,7 @@ static const struct sim_eons_operator_info *
 			break;
 	}
 
-	if (!l)
+	if (l == NULL)
 		return NULL;
 
 	opl = l->data;
@@ -1256,8 +1257,6 @@ void sim_adn_build(unsigned char *data, int length,
 
 	number_len = (number_len + 1) / 2;
 	*data++ = number_len + 1;
-
-	/* Use given number type and 'Unknown' for Numbering Plan */
 	*data++ = ph->type;
 
 	sim_encode_bcd_number(ph->number, data);
@@ -1315,7 +1314,7 @@ gboolean sim_parse_3g_get_response(const unsigned char *data, int len,
 	 */
 	tlv = ber_tlv_find_by_tag(fcp, 0x80, fcp_length, &tlv_length);
 
-	if (!tlv || tlv_length < 2)
+	if (tlv == NULL || tlv_length < 2)
 		return FALSE;
 
 	flen = tlv[0];
@@ -1324,14 +1323,14 @@ gboolean sim_parse_3g_get_response(const unsigned char *data, int len,
 
 	tlv = ber_tlv_find_by_tag(fcp, 0x83, fcp_length, &tlv_length);
 
-	if (!tlv || tlv_length != 2)
+	if (tlv == NULL || tlv_length != 2)
 		return FALSE;
 
 	id = (tlv[0] << 8) | tlv[1];
 
 	tlv = ber_tlv_find_by_tag(fcp, 0x82, fcp_length, &tlv_length);
 
-	if (!tlv || (tlv_length != 2 && tlv_length != 5))
+	if (tlv == NULL || (tlv_length != 2 && tlv_length != 5))
 		return FALSE;
 
 	if (tlv[1] != 0x21)
@@ -1380,7 +1379,7 @@ gboolean sim_parse_3g_get_response(const unsigned char *data, int len,
 
 	acc[2] = 0x44;
 
-	if (!info)
+	if (info == NULL)
 		acc[0] = 0x11;
 	else
 		acc[0] = (info->perm_read << 4) | info->perm_update;
@@ -1405,7 +1404,8 @@ gboolean sim_parse_3g_get_response(const unsigned char *data, int len,
 
 gboolean sim_parse_2g_get_response(const unsigned char *response, int len,
 					int *file_len, int *record_len,
-					int *structure, unsigned char *access)
+					int *structure, unsigned char *access,
+					unsigned char *file_status)
 {
 	if (len < 14 || response[6] != 0x04)
 		return FALSE;
@@ -1420,6 +1420,8 @@ gboolean sim_parse_2g_get_response(const unsigned char *response, int len,
 	access[1] = response[9];
 	access[2] = response[10];
 
+	*file_status = response[11];
+
 	if (response[13] == 0x01 || response[13] == 0x03)
 		*record_len = response[14];
 	else
